﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.ComponentModel;
using System.Collections.ObjectModel;
using Buzz.MachineInterface;
using BuzzGUI.Interfaces;
using BuzzGUI.Common;
using EpForceDirectedGraph.cs;

namespace WDE.MachineViewOrganizer
{
	[MachineDecl(Name = "Machine View Organizer", ShortName = "Organizer", Author = "wde", MaxTracks = 1)]
	public class MachineViewOrganizerMachine : IBuzzMachine, INotifyPropertyChanged
	{
		IBuzzMachineHost host;
        Graph m_fdgGraph = new Graph();

        public MachineViewOrganizerMachine(IBuzzMachineHost host)
		{
			this.host = host;                      
        }

        [ParameterDecl(ValueDescriptions = new[] { "No", "Yes" }, DefValue = 0)]
        public bool RandomizePos { get; set; }

        [ParameterDecl(MinValue = 0, MaxValue = 10000, DefValue = 4000)]
        public int Zoom { get; set; }

        [ParameterDecl(MinValue = 0, MaxValue = 8000, DefValue = 800)]
        public int Stiffness { get; set; }

        [ParameterDecl(MinValue = 0, MaxValue = 64000, DefValue = 40000)]
        public int Repulsion { get; set; }

        [ParameterDecl(MinValue = 0, MaxValue = 500, DefValue = 50)]
        public int Damping { get; set; }

        [ParameterDecl(MinValue = 0, MaxValue = 100, DefValue = 10)]
        public int Threshold { get; set; }

        [ParameterDecl(MinValue = 0, MaxValue = 10000, DefValue = 100)]
        public int Iterations { get; set; }

        [ParameterDecl(MinValue = 0, MaxValue = 300, DefValue = 30)]
        public int Mass { get; set; }

        [ParameterDecl(MinValue = 0, MaxValue = 6000, DefValue = 600)]
        public int EdgeLength { get; set; }

        [ParameterDecl(MinValue = 0, MaxValue = 10000, DefValue = 100)]
        public int IterationCount { get; set; }

        [ParameterDecl(MinValue = 50, MaxValue = 500, DefValue = 260)]
        public int SnapStepX { get; set; }

        [ParameterDecl(MinValue = 50, MaxValue = 500, DefValue = 200)]
        public int SnapStepY { get; set; }

        // actual machine ends here. the stuff below demonstrates some other features of the api.

        public class State : INotifyPropertyChanged
		{
			public State() { text = "here is state"; }	// NOTE: parameterless constructor is required by the xml serializer

			string text;
			public string Text 
			{
				get { return text; }
				set
				{
					text = value;
					if (PropertyChanged != null) PropertyChanged(this, new PropertyChangedEventArgs("Text"));
					// NOTE: the INotifyPropertyChanged stuff is only used for data binding in the GUI in this demo. it is not required by the serializer.
				}
			}

			public event PropertyChangedEventHandler PropertyChanged;
		}

		State machineState = new State();
		public State MachineState			// a property called 'MachineState' gets automatically saved in songs and presets
		{
			get { return machineState; }
			set
			{
				machineState = value;
				if (PropertyChanged != null) PropertyChanged(this, new PropertyChangedEventArgs("MachineState"));
			}
		}		
	
		public IEnumerable<IMenuItem> Commands
		{
			get
			{
				yield return new MenuItemVM() 
				{ 
					Text = "About...", 
					Command = new SimpleCommand()
					{
						CanExecuteDelegate = p => true,
						ExecuteDelegate = p => MessageBox.Show("Neatness maximus. \n\r\n\rUsing EpForceDirectedGraph.cs https://github.com/juhgiyo/EpForceDirectedGraph.cs")
					}
				};
			}
		}		

		public event PropertyChangedEventHandler PropertyChanged;

        class Renderer : AbstractRenderer
        {
            IMachineGraph graph;
            MachineViewOrganizerMachine neatMac;

            public Renderer(IForceDirected iForceDirected, IMachineGraph graph, MachineViewOrganizerMachine neatMac) : base(iForceDirected)
            {
                this.graph = graph;
                this.neatMac = neatMac;
                
            }

            public override void Clear()
            {
                // Clear previous drawing if needed
                // will be called when AbstractRenderer:Draw is called
            }

            protected override void drawEdge(Edge iEdge, AbstractVector iPosition1, AbstractVector iPosition2)
            {
                // Draw the given edge according to given positions
            }

            protected override void drawNode(Node iNode, AbstractVector iPosition)
            {
                // Draw the given node according to given position
                foreach (IMachine m in graph.Machines)
                {
                    if (m.Name == iNode.Data.label)
                    {
                        FDGVector2 vec = (FDGVector2)iPosition;
                        float zoomi = neatMac.Zoom / 10.0f;
                        var coords = new Tuple<float, float>((vec.x)/zoomi, (vec.y)/zoomi);
                        var info = new Tuple<IMachine, Tuple<float,float>>(m, coords);

                        Collection<Tuple<IMachine, Tuple<float, float>>> dirtyMachines = new Collection<Tuple<IMachine, Tuple<float, float>>>();
                        dirtyMachines.Add(info);
                        graph.MoveMachines(dirtyMachines);
                    }
                }
            }
        };

        internal void MakeGUINice()
        {            
            m_fdgGraph = new Graph();
            
            foreach (IMachine m in host.Machine.Graph.Machines)
            {
                NodeData data = new NodeData();
                data.mass = Mass / 10.0f;
                data.label = m.Name;
                FDGVector2 vec = new FDGVector2(m.Position.Item1 * EdgeLength / 10.0f, m.Position.Item2 * EdgeLength / 10.0f);
                if (!RandomizePos) data.initialPostion = vec;
                Node newNode = new Node(m.Name, data);

                m_fdgGraph.AddNode(newNode);        
            }

            foreach (IMachine m in host.Machine.Graph.Machines)
            {                
                foreach (IMachineConnection conn in m.Inputs)
                {
                    IMachine source_m = conn.Source;

                    Node node1 = m_fdgGraph.GetNode(m.Name);
                    Node node2 = m_fdgGraph.GetNode(source_m.Name);

                    EdgeData data = new EdgeData();
                    data.label = m.Name + "-" + source_m.Name;
                    data.length = EdgeLength / 10.0f;                    
                    Edge newEdge = m_fdgGraph.CreateEdge(node2, node1, data);
                }
                
            }

            float stiffness = Stiffness / 10.0f;
            float repulsion = Repulsion;
            float damping = Damping / 100.0f;

            // 2D Force Directed
            ForceDirected2D m_fdgPhysics = new ForceDirected2D(m_fdgGraph, // instance of Graph
                                                               stiffness, // stiffness of the spring
                                                               repulsion, // node repulsion rate 
                                                               damping    // damping rate  
                                                               );
            m_fdgPhysics.Threadshold = Threshold / 100.0f;

            Renderer m_fdgRenderer = new Renderer(m_fdgPhysics, host.Machine.Graph, this);

            float timeStep = 0.05f; // The time passed from previous Draw call in second
            for (int i = 0; i < IterationCount; i++)
                m_fdgRenderer.Draw(timeStep);
                        
        }

        internal void SnapToGrid()
        {            
            // Draw the given node according to given position
            foreach (IMachine m in host.Machine.Graph.Machines)
            {
                float stepX = SnapStepX;
                float stepY = SnapStepY;
                float x = m.Position.Item1 * 1000;
                float y = m.Position.Item2 * 1000;
                float newX = stepX * (float)Math.Floor((x / stepX) + 0.5) / 1000.0f;
                float newY = stepY * (float)Math.Floor((y / stepY) + 0.5) / 1000.0f;

                var coords = new Tuple<float, float>(newX, newY);
                var info = new Tuple<IMachine, Tuple<float, float>>(m, coords);

                Collection<Tuple<IMachine, Tuple<float, float>>> dirtyMachines = new Collection<Tuple<IMachine, Tuple<float, float>>>();
                dirtyMachines.Add(info);
                host.Machine.Graph.MoveMachines(dirtyMachines);
            }
        }
    }

	public class MachineGUIFactory : IMachineGUIFactory { public IMachineGUI CreateGUI(IMachineGUIHost host) { return new TidyGUI(); } }
	public class TidyGUI : UserControl, IMachineGUI
	{
		IMachine machine;
		MachineViewOrganizerMachine tidyMachine;
        Button btTidy;
        Button btSnapToGrid;

        // view model for machine list box items
        public class MachineVM
		{
			public IMachine Machine { get; private set; }
			public MachineVM(IMachine m) { Machine = m; }
			public override string ToString() { return Machine.Name; }
		}
		
		public ObservableCollection<MachineVM> Machines { get; private set; }
		
		public IMachine Machine
		{
			get { return machine; }
			set
			{
				machine = value;

				if (machine != null)
				{
					tidyMachine = (MachineViewOrganizerMachine)machine.ManagedMachine;
				}
			}
		}
        		
		public TidyGUI()
		{
			Machines = new ObservableCollection<MachineVM>();

            btTidy = new Button();
            btTidy.Content = new TextBlock() { Text = "Use Force Algorithm", FontSize= 18 };

            btTidy.Click += BtTidy_Click;

            btSnapToGrid = new Button();
            btSnapToGrid.Content = new TextBlock() { Text = "Snap To Grid", FontSize = 18 };
            btSnapToGrid.Click += BtSnapToGrid_Click;

            var sp = new StackPanel();
            sp.Children.Add(btTidy);
            sp.Children.Add(btSnapToGrid);
            //sp.Children.Add(tb);
            //sp.Children.Add(lb);
            this.Content = sp;
		
		}

        private void BtSnapToGrid_Click(object sender, RoutedEventArgs e)
        {
            if (tidyMachine != null)
                tidyMachine.SnapToGrid();
        }

        private void BtTidy_Click(object sender, RoutedEventArgs e)
        {
            if (tidyMachine != null)
                tidyMachine.MakeGUINice();
        }
    }

}
